//	StringParser.cpp - Simple parser class to read from a string and output the
//					   digits from it.
//  ----------------------------------------------------------------------------
//	This file is part of Motion Composition Tool, a simple application for
//	creating simple abstract animations.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//	----------------------------------------------------------------------------

#include "StringParser.h"

#include <sstream>
#include <iostream>

using namespace std;

//------------------------------------------------------------------------------
StringParser::StringParser(const string& input)
{
	fillNumbers(input);
}

//------------------------------------------------------------------------------
StringParser::~StringParser()
{
	
}

//------------------------------------------------------------------------------
int StringParser::getNumFloats() const
{
	return numbers.size();
}

//------------------------------------------------------------------------------
int StringParser::getNumFourFloats() const
{
	int retval = 0;

	if((numbers.size() % 4) == 0)
		retval = numbers.size()/4;

	return retval;
}

//------------------------------------------------------------------------------
int StringParser::getNumEightFloats() const
{
	int retval = 0;

	if((numbers.size() % 12) == 0)
		retval = numbers.size()/12;

	return retval;
}

//------------------------------------------------------------------------------
float StringParser::getFloat(int index) const
{
	return numbers[index];
}

//------------------------------------------------------------------------------
FourFloats StringParser::getFourFloats(int index) const
{
	FourFloats retval;

	//Check that the string could have held a multiple of FourFloats first.
	if((numbers.size() % 4) == 0)
	{
		retval.r = numbers[index*4];
		retval.x = numbers[(index*4)+1];
		retval.y = numbers[(index*4)+2];
		retval.z = numbers[(index*4)+3];
	}

	return retval;
}

//------------------------------------------------------------------------------
EightFloats StringParser::getEightFloats(int index) const
{
	EightFloats retval;

	//Check that the string could have held a multiple of FourFloats first.
	if((numbers.size() % 12) == 0)
	{
		retval.x[0] = numbers[index*12];
		retval.y[0] = numbers[(index*12)+1];
		retval.x[1] = numbers[(index*12)+3];
		retval.y[1] = numbers[(index*12)+4];
		retval.x[2] = numbers[(index*12)+6];
		retval.y[2] = numbers[(index*12)+7];
		retval.x[3] = numbers[(index*12)+9];
		retval.y[3] = numbers[(index*12)+10];
	}

	return retval;
}

//------------------------------------------------------------------------------
void StringParser::fillNumbers(const string& input)
{
	double tempd;
	string tempstr;
	string tempstr2;
	int tempint = 0;
	int tempHex = -1;

	//First check if the string is empty.
	if(input.empty())
	{
		cout << "StringParser: string is empty." << endl;
		return;
	}
	else if((input.find_first_of(" \t\r\n") == 0) && (input.size() == 1))
	{
		cout << "StringParser: string is effectively empty." << endl;
		return;
	}

	//Next check if the string holds a single number.
	if(input.find_first_of(" \t\r\n") == string::npos) 
	{
		stringstream tempConverter;

		tempConverter << input;
		tempConverter >> tempd;

		numbers.push_back(tempd);
	}
	else //There's a stream of numbers to be parsed.
	{
		if(input.find_first_of(" \t\r\n") != 0)
		{
			tempstr = input.substr(tempint, (input.find_first_of(" \t\r\n")));
			tempstr2 = input.substr(input.find_first_of(" \t\r\n")+1);
		}
		else
		{
			tempstr2 = input.substr(1);
			tempstr = tempstr2.substr(tempint, (tempstr2.find_first_of(" \t\r\n")));
			tempstr2 = tempstr2.substr(tempstr2.find_first_of(" \t\r\n")+1);
		}

		while(static_cast<unsigned int>(tempint) < input.size())
		{
			stringstream tempConverter;

			//To watch for multiple whitespace characters.
			while((tempstr.size() == 0)&&(static_cast<unsigned int>(tempint) < input.size()))
			{
				tempint += tempstr.size()+1;

				tempstr = tempstr2.substr(0, (tempstr2.find_first_of(" \t\r\n")));
				tempstr2 = tempstr2.substr(tempstr2.find_first_of(" \t\r\n")+1);
			}

			tempConverter << tempstr;
			tempConverter >> tempd;

			numbers.push_back(tempd);

			tempint += tempstr.size()+1;

			tempstr = tempstr2.substr(0, (tempstr2.find_first_of(" \t\r\n")));
			tempstr2 = tempstr2.substr(tempstr2.find_first_of(" \t\r\n")+1);

			tempd = -1.0;
		}
	}
}
